; Andrew Noymer
;   version 0.1, August 2010
;   version 0.2, July 2013, edited for code clarity
; function to return Goodman-Grunfeld
; test statistic from 2 vectors
;
; if the vectors passed are not the same length, NaN
; is returned.  That's not the most sophisticated 
; error checking in the history of the world, but it's
; a start.
;
; could add an error check vis-a-vis verification that n
; is equal to the length of the original array, minus one. 
;
; KNOWN inefficiencies:
;
; the j loop could possibly be replaced by a vector
; operation
;
; the k loop would be more efficient with nested Boolean evals.
;
; conversions to double is to avoid IDL from doing things
; as integer operations, which it has a tendency to do
; (e.g. 3/6=0 [integer division] but lots of times we are
;  expecting 3/6=0.5000)... but this may have been overkill
; here - I did not look to see if would have been a problem without
;
; /end inefficiencies;
;
;
; COPYRIGHT NOTICE
;
; Copyright (c) 2010-2014 Andrew Noymer
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

FUNCTION goodman_grunfeld, arr1, arr2, continuity=continuity 
; "continuity" is an optional keyword for whether
; the continuity correction is applied
AA = !VALUES.F_NAN 
BB = 1
if N_elements(arr1) NE N_elements(arr2) then begin
    BB=0
endif else begin
   n=N_elements(arr1)
   length=n-1
   U=bytarr(length)
   V=bytarr(length)
   for j=0,length-1 do begin
      if arr1[j+1] GT arr1[j]  then U[j]=1
      if arr2[j+1] GT arr2[j]  then V[j]=1
   endfor
   ; for data sets with >=32,767 observations, change "fix" to "long"
   a=fix(0)
   b=fix(0)
   c=fix(0)
   d=fix(0)
   ; I find that nested if..then..else in IDL 
   ; confuses me, so I do this somewhat more
   ; brutally than necessary... fix eventually
   for k=0,length-1 do begin
      if (U[k] EQ 1) AND (V[k] EQ 1) then ++a
      if (U[k] EQ 1) AND (V[k] EQ 0) then ++b        
      if (U[k] EQ 0) AND (V[k] EQ 1) then ++c
      if (U[k] EQ 0) AND (V[k] EQ 0) then ++d                 
   endfor
   n=a+b+c+d
   e=double(0.0)
   f=double(0.0)
   nprime=n-1
   maxloop=nprime-1
   for i=0,maxloop do begin
         e=e+U[i]*U[i+1]
         f=f+V[i]*V[i+1]
      endfor
   ; convert to floating-point because of the way IDL handles types
   a=double(a)
   b=double(b)
   c=double(c)
   d=double(d)
   NN=a+b+c+d
   np=double(nprime)
   e= (e/np)-((a+b)/NN)^2
   f= (f/np)-((a+c)/NN)^2   
endelse
if BB EQ 1 then begin
if (keyword_set(continuity)) then begin  
     AA = sqrt(NN)*(abs(a*d-b*c)-0.5*NN)/sqrt((a+b)*(c+d)*(a+c)*(b+d)+2*(NN^4)*e*f)
  endif else begin
     AA = sqrt(NN)*(abs(a*d-b*c))/sqrt((a+b)*(c+d)*(a+c)*(b+d)+2*(NN^4)*e*f)
  endelse
endif
; RETURN, [a,b,c,d,temp_e,temp_f]
RETURN, AA
END
