pro run_gg

;
; code to test for comovement of e(0) between the sexes 
; for a battery of HMD countries... 
;

; Andrew Noymer & Viola Van

; make sure numlines is an integer not a float, by declaring it such:
numlines=0

; one-sided test:
crit_value=1.64485359


; make the first one an empty string, since I want to
; count files 1,2,3 not 0,1,2
filenames = [ '', $
'australia_hmd.raw', $
'austria_hmd.raw', $
'belarus_hmd.raw', $
'belgium_new_2.raw', $ 
'bulgaria_hmd.raw', $
'canada_hmd.raw', $
'chile_hmd.raw', $
'czech_hmd.raw', $
'danmark_hmd.raw', $
'estonia_hmd.raw', $
'finland_hmd.raw', $
'france_hmd.raw', $
'gdr_hmd.raw', $
'frg_hmd.raw', $
'hungary_hmd.raw', $
'ireland_hmd.raw', $
'island_hmd.raw', $
'israel_hmd.raw', $
'italy_hmd.raw', $
'japan_hmd.raw', $
'latvia_hmd.raw', $
'lithuania_hmd.raw', $
'luxemburg_hmd.raw', $
'netherlands_hmd.raw', $
'newzealand_hmd.raw', $
'norway_hmd.raw', $
'poland_hmd.raw', $
'portugal_hmd.raw', $
'russia_hmd.raw', $
'slovakia_hmd.raw', $
'slovenia_hmd.raw', $
'spain_hmd.raw', $
'sweden_hmd.raw', $
'switzerland_hmd.raw', $
'taiwan_hmd.raw', $
'englandandwales_hmd.raw', $
'scotland_hmd.raw', $
'northernireland_hmd.raw', $
'ukraine_hmd.raw', $
'USA_hmd.raw' ]

dimension=N_elements(filenames)
num_files=dimension-1

; allocate empty pointer array
; see Gumley p. 61 for discussion of this
metadata=ptrarr(N_elements(filenames), /Allocate_Heap)
metadata_year=ptrarr(N_elements(filenames), /Allocate_Heap)
;;
;; useful "cheat sheets" on IDL pointers:
;;    http://www.dfanning.com/misc_tips/pointers.html
;;    http://www.dfanning.com/code_tips/ptrarray.html
;;

for file = 1,num_files do begin
  get_lun, LUN
  openr, LUN, filenames(file)
  readf, LUN, numlines
  data  = dblarr(4,numlines) ;; data array from input file
  readf, LUN, data
  close, LUN
  free_lun, LUN
  ; for the time being, we want only the sexes...
  *metadata[file]=[data(1,*),data(2,*)]
  ; for the divergence exercise we are interestd in the year:  
  *metadata_year[file]=[data(0,*)]
endfor

; store results in an array:
results=dblarr(2,dimension)

; now do the GG calculations as a loop

FOR j = 1,num_files do begin
    a=*metadata[j]
    arr1=a(0,*)
    arr2=a(1,*)
    ;; print, transpose(arr1)
    ;; print, transpose(arr2)
    GGcc=goodman_grunfeld(arr1,arr2,/continuity)
    if (GGcc GE crit_value) AND (GGcc NE !VALUES.F_INFINITY) then BEGIN
             print, "* ",filenames(j) 
          endif 
    ; we are more interested if they are NOT significant, so:
    if (GGcc LT crit_value) AND (GGcc NE -!VALUES.F_INFINITY) then BEGIN
             print, "NS ",filenames(j), GGcc 
          endif 
    results(0,j)=GGcc 
ENDFOR


print
print, ' %%%%%%%% second part '
print


;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;; part 2

;; Here we try to avoid the problem (or perceived or potential problem)
;; of having significant GG statistics for co-movement due to a long
;; history... for example if we have 150 years of co-movement followed
;; by 30 years of divergence we don't want the first part to counterweight
;; the second part.  So we go thru each country with a moving wall and
;; find the 25 years of maximum-divergence, and then run that for each
;; country.



num_times_sig=0
num_times_NS=0

magnitude_diff=dblarr(num_files+1)
start_pos_rec=dblarr(num_files+1)

FOR j = 1,num_files do begin
    a=*metadata[j]
    b=*metadata_year[j]
    arr1=a(0,*)
    arr2=a(1,*)
    ; sex difference is this:
    diff=arr1-arr2
    ; for all possible starting values,
    ; find the moving wall with maximum divergence
    start_pos = 0
    wall_size=25 ; years
    IF N_ELEMENTS(diff) GT wall_size then begin
      max_diff = 0.0
      FOR start = 0,(N_ELEMENTS(diff)-wall_size) do begin
         if abs(diff(start)-diff(start+wall_size-1)) GT max_diff then begin
           start_pos = start
           max_diff = abs(diff(start)-diff(start+wall_size-1))
        endif ; abs
      endfor ; start
    endif ; >wall_size
    ;; in the event of ties, the above loop implicitly takes
    ;; the earlier start_pos...
    ; save for later:
    magnitude_diff(j)=max_diff    
    start_pos_rec(j)=b(start_pos)
    ; now redefine wall_size to take into account
    ; countries with less data than the moving wall itself
    wall_size=min([N_Elements(diff),wall_size])
    arr1=a(0,start_pos:start_pos+wall_size-1)
    arr2=a(1,start_pos:start_pos+wall_size-1)
    year=*metadata_year[j]
    GGcc=goodman_grunfeld(arr1,arr2,/continuity)
    if (GGcc GE crit_value) AND (GGcc NE !VALUES.F_INFINITY) then BEGIN
            num_times_sig++
          endif 
    ; we are more interested if they are NOT significant, so:
    if (GGcc LT crit_value) AND (GGcc NE -!VALUES.F_INFINITY) then BEGIN
            num_times_NS++
          endif 
    results(1,j)=GGcc 
 ENDFOR ; j

print, 'sig: ', num_times_sig, ' NS: ', num_times_NS




;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;;
;; part 3

print
print, ' %%%%%%%% third part '
print

;; make a table of descriptive stats 

;; here is an example of what we want:

;; item 1: country name
;; item 2: country begin date
;; item 3: country end date
;; item 4: Male e(0) min
;; item 5: date of item 4
;; item 6: Male e(0) max
;; item 7: date of item 6
;; item 8-11: same as 4-7, for Female
;; item 12: min of F-M diff
;; item 13: date of item 12
;; item 14: max of F-M diff
;; item 15: date of item 14
;; item 16: magnitude of 25-year divergence
;; item 17: start date of item 16

output_descriptives=dblarr(17+1,num_files+1)

; item 1:
country_names = [ '', $
'Australia', $
'Austria', $
'Belarus', $
'Belgium', $ 
'Bulgaria', $
'Canada', $
'Chile', $
'Czech Republic', $
'Denmark', $
'Estonia', $
'Finland', $
'France', $
'GDR', $
'FRG', $
'Hungary', $
'Ireland', $
'Iceland', $
'Israel', $
'Italy', $
'Japan', $
'Latvia', $
'Lithuania', $
'Luxemburg', $
'Netherlands', $
'New Zealand', $
'Norway', $
'Poland', $
'Portugal', $
'Russia', $
'Slovakia', $
'Slovenia', $
'Spain', $
'Sweden', $
'Switzerland', $
'Taiwan', $
'England and Wales', $
'Scotland', $
'Northern Ireland', $
'Ukraine', $
'United States' ]

; item 2

FOR j = 1,num_files do begin
    a=*metadata_year[j]
 ;;   print, a(0) ;; same as:  min(a(0,*))
    output_descriptives(2,j)=a(0)
ENDFOR


; item 3

FOR j = 1,num_files do begin
    a=*metadata_year[j]
    ;; print, a(0,0) ;; same as:  min(a(0,*))
    ;; in IDL 8.0 or greater can just use "-1" instead of Q
    Q=N_ELEMENTS(a)-1
    output_descriptives(3,j)=a(0,Q)
ENDFOR


; item 4  --- male min( e(0) )

FOR j = 1,num_files do begin
    a=*metadata[j]
;;    print, min(a(1,*))
    output_descriptives(4,j)=min(a(1,*))
ENDFOR

; item 5 --- date of item 4

;; in the case of ties, take the LATEST date
;; we can search for the presence of ties later on
;; but for now on, use this convention...

FOR j = 1,num_files do begin
    a=*metadata[j]
    b=*metadata_year[j]
    target=min(a(1,*))
    Q=N_ELEMENTS(a(1,*));-1
    REPEAT Q=Q-1 UNTIL a(1,Q) EQ target 
    output_descriptives(5,j)=b(Q)
;;    print, output_descriptives(4,j), output_descriptives(5,j) 
ENDFOR

; item 6  --- male max( e(0) )

FOR j = 1,num_files do begin
    a=*metadata[j]
;;    print, max(a(1,*))
    output_descriptives(6,j)=max(a(1,*))
ENDFOR

; item 7 --- date of item 6

;; in the case of ties, take the EARLIEST date
;; we can search for the presence of ties later on
;; but for now on, use this convention...

FOR j = 1,num_files do begin
    a=*metadata[j]
    b=*metadata_year[j]
    target=max(a(1,*))
    Q=0
    REPEAT Q=Q+1 UNTIL a(1,Q) EQ target 
    output_descriptives(7,j)=b(Q)
;    print, output_descriptives(6,j), output_descriptives(7,j) 
ENDFOR


; item 8  --- female min( e(0) )

FOR j = 1,num_files do begin
    a=*metadata[j]
;;    print, min(a(1,*))
    output_descriptives(8,j)=min(a(0,*))
ENDFOR

; item 9 --- date of item 8

FOR j = 1,num_files do begin
    a=*metadata[j]
    b=*metadata_year[j]
    target=min(a(0,*))
    Q=N_ELEMENTS(a(0,*));-1
    REPEAT Q=Q-1 UNTIL a(0,Q) EQ target 
    output_descriptives(9,j)=b(Q)
ENDFOR

; item 10  --- female max( e(0) )

FOR j = 1,num_files do begin
    a=*metadata[j]
;;    print, max(a(1,*))
    output_descriptives(10,j)=max(a(0,*))
ENDFOR

; item 11 --- date of item 10

FOR j = 1,num_files do begin
    a=*metadata[j]
    b=*metadata_year[j]
    target=max(a(0,*))
    Q=0
    REPEAT Q=Q+1 UNTIL a(0,Q) EQ target 
    output_descriptives(11,j)=b(Q)
;;    print, output_descriptives(10,j), output_descriptives(11,j) 
ENDFOR

; item 12 & 14  --- min/max ( f - m diff )

FOR j = 1,num_files do begin
    a=*metadata[j]
;;    print, max(a(1,*))
    diff = a(0,*)-a(1,*)
    output_descriptives(12,j)=min(diff)
    output_descriptives(14,j)=max(diff)
ENDFOR

; item 13 --- date of item 12

; for ties, find most-recent

FOR j = 1,num_files do begin
    a=*metadata[j]
    b=*metadata_year[j]
    diff = a(0,*)-a(1,*)
    target=output_descriptives(12,j)
    Q=N_ELEMENTS(a(1,*));-1
    REPEAT Q=Q-1 UNTIL diff(Q) EQ target 
    output_descriptives(13,j)=b(Q)
;;    print, output_descriptives(4,j), output_descriptives(5,j) 
ENDFOR

; item 14 --- date of item 13

; for ties, find oldest

FOR j = 1,num_files do begin
    a=*metadata[j]
    b=*metadata_year[j]
    diff = a(0,*)-a(1,*)
    target=output_descriptives(14,j)
    Q=0
    REPEAT Q=Q+1 UNTIL diff(Q) EQ target 
    output_descriptives(15,j)=b(Q)
;    print, output_descriptives(6,j), output_descriptives(7,j) 
ENDFOR

output_descriptives(16,*)=magnitude_diff
output_descriptives(17,*)=start_pos_rec

openw, lun, 'table_data.txt', /get_lun, width=300
   printf, lun, output_descriptives(2:17,1:*)
free_LUN, lun


;; to-do: check these for the presence of ties...
;; most probably have no-ties but we should check...

openw, lun, 'table_one_base.tex', /get_lun, width=300
   printf, lun, output_descriptives(2:11,1:*), $
  FORMAT='(" &",I5," &",I5," &",F8.2," &",I5," &",F8.2," &",I5," && ",F8.2," &",I5," &",F8.2," &",I5,"\\")'
free_LUN, lun

openw, lun, 'table_two_base.tex', /get_lun, width=300
   printf, lun, output_descriptives(12:17,1:*), $
  FORMAT='(" &",F8.2," &",I5," &&",F8.2," &",I5," &&",F8.2," &",I5," \\")'
free_LUN, lun


openw, lun, 'country_names.txt', /get_lun, width=50
for j=1,num_files do begin
   printf, lun, country_names(j)
endfor
free_LUN, lun

openw, lun, 'GG_results.txt', /get_lun, width=50
for j=1,num_files do begin
   printf, lun, country_names[j], results(*,j)
endfor
free_LUN, lun


 
END
